function[Satisfied, Edges, et, TotalSat, TotalLam,t, Worked]=SimpleBackward(Lambda,Links,NoOfSteps)

Small=0.000000001;
Infi=10000000000;
eqCount=0;
N=length(Lambda);
E=length(Links);

%---------------------------------------------%
%---------------------------------------------%

%Rewrite the edges as adjacency matrices
[NeighbourM, NeighbourCM, NeighbourICon]=zMakeNeighbourMatrix(N, Links);

%---------------------------------------------%
%Starting Matrices for the values of A, B, and Y
i=max(NeighbourICon);
A=zeros(N,i);
B=ones(N,i);
Y=zeros(N,i);

[Randi]=zRandomlyChooseJ(Links,NoOfSteps,N);
 tic;
%Start Iterations!
for t=1:NoOfSteps*N
    
    %Assining Nodes J and I
    nodeJ=Randi(1,t); %Node J
    connectivity=NeighbourICon(nodeJ);%Connectivity of node J
    nodeI=Randi(2,t); %Node I
    cavityI=find(NeighbourM(nodeJ,:)==nodeI);%Where I is on J's adjacency matrix
    cavityJ=find(NeighbourM(nodeI,:)==nodeJ);%Where J is on I's adjacency matrix
    
    %Rewriting for ease
    Yij=Y(nodeI,cavityJ);
    Yji=Y(nodeJ,cavityI);
    Aij=A(nodeI,cavityJ);
    Aji=A(nodeJ,cavityI);
    Bij=B(nodeI,cavityJ);
    Bji=B(nodeJ,cavityI);
    mu=Lambda(nodeJ);
    Phiij1=Y(nodeI,cavityJ);%Assuming quadratic transportation cost
    Phiij2=1;%Assuming quadratic transportation costs w. no resistance
    
    %Fetching values of A, B and Y from descendants k for J
    for k=1:connectivity
        Ajk(k)=A(nodeJ,k);
        Bjk(k)=B(nodeJ,k);
        Yjk(k)=Y(nodeJ,k);
        Phijk1(k)=Y(nodeJ,k); %Quadratic no resistance
        Phijk2(k)=1; %Quadratic no resistance
    end
    
    %Summing the descendants for sum(y-a/b) and sum(1/b) in mu
    tempd1=0;
    tempd2=0;
    
    for k=1:connectivity
        if k~=cavityI
            tempd1=tempd1+Yjk(k)-((Ajk(k)+Phijk1(k))/(Bjk(k)+Phijk2(k)));
            tempd2=tempd2+(1/(Bjk(k)+Phijk2(k)));
        end
    end
    
    %Calculating Mu=min(0, mu-x/tempd2)
    x=Yij-tempd1;%Just for simplification later!
    
    Pos=0;%If Pos=0 then mu is negative or zero, if Pos=1, it was postive but make to be equal 0.
    tempmu=(mu-x)/tempd2;
    if tempmu>0
        Pos=1;
        tempmu=0;
    end
    
    %Messages before to check for convergence after
    preA=Aij;
    preB=Bij;
    for k=1:connectivity
        if k~=cavityI
            preY(k)=Yjk(k);
        end
    end
    
    %Calculating new Aij 
    A(nodeI,cavityJ)=-tempmu;
    Aij=A(nodeI,cavityJ);
    
    %Calculating new Bij
    if Pos==0
        B(nodeI,cavityJ)=1/tempd2;
    else
        B(nodeI,cavityJ)=0;
    end
    Bij=B(nodeI,cavityJ);
    
    %Calculating the new backwards Yjk messages!
    for k=1:connectivity
        if k~=cavityI
            Y(nodeJ,k)=Yjk(k)-(Ajk(k)+Phijk1(k)+tempmu)/(Bjk(k)+Phijk2(k));
            Yjk(k)=Y(nodeJ,k);
        end
    end
    
     
    %Checking if the network has converged
    fieldChange=abs(Aij-preA)+abs(Bij-preB);%Has anything changed?
    for k=1:connectivity
        if k~=cavityI
            fieldChange=fieldChange+abs(Yjk(k)-preY(k));
        end
    end
    
    if fieldChange<Small
        eqCount=eqCount+1;%might of converged, need to check the rest!
    else
        eqCount=0;%hasn't converged
    end
    
    %Finish if converged
    if eqCount>E*4 %E*2 but just to be sure.
        fprintf('Break at %5.0f Iterations \n',t);
        break
    end
    
end
et=toc;
%---------------------------------------------%
%---------------------------------------------%

%Chance adjacency matrices back into edges
[Edges]=zShowYAsEdgesEdges(Links, Y, NeighbourM, NeighbourCM, Small);

%Calculate how much power each node has after convergence
[Satisfied, TotalSat, TotalLam]=zCheck(Lambda, Y, NeighbourICon, Small);

%Calculate how much power each node has after convergence
Worked=sum(abs(Edges(:,3)-Edges(:,4)))>0.001;
Worked=Worked+heaviside(sum(isnan(Edges(:,3)))-0.001);
Worked=Worked+heaviside(sum(Satisfied<-0.001)-0.00001);
Worked=Worked+(abs(TotalLam-TotalSat)>0.001);
end
